/************************************************************
	FILE:		lib_image.c
	CONTAINS:	functions for reading and writing SGI RGB-format images
************************************************************/

#include "os.h"
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>

#include "lib_defs.h"
#include "lib_globals.h"
#include "lib_image.h"

/////////////////////////////////////////
//	PROC:	save_image
//	DOES:	saves the current image to a ppm file
/////////////////////////////////////////

void save_image()
{
  FILE *fp;
  STR fname;
  const int maxVal=255;
  register int y;
  unsigned char *pixels;

  sprintf(fname,"%s%d", Image_name, Image_num);
  printf("Saving image %s\n", fname);
  Image_num++;
  fp = fopen(fname,"w");
  if (!fp) {
	printf("Unable to open file '%s'\n",fname);
	return;
  }
  fprintf(fp, "P6\n");
  fprintf(fp, "%d %d\n", Win[0], Win[1]);
  fprintf(fp, "%d\n", maxVal);

  pixels = new unsigned char [3*Win[0]];
  for ( y = Win[1]-1; y>=0; y-- ) {
	glReadPixels(0,y,Win[0],1,GL_RGB,GL_UNSIGNED_BYTE, (GLvoid *) pixels);
    fwrite(pixels, 3, Win[0], fp);
  }
  fclose(fp);
}

/*********************************************************
    PROC:  mymalloc()
    DOES:  allocates memory
**********************************************************/

char *mymalloc(int size)
{
  char *buf;

  buf = (char *) malloc(size);
  if (!buf) {
    fprintf(stderr,"malloc() failure\n");
    exit(-1);
  }
  return(buf);
}

/*********************************************************
    PROC:  getbyte()
    DOES: 
**********************************************************/

unsigned char getbyte(FILE *inf)
{
  unsigned char buf;

  fread(&buf,1,1,inf);
  return(buf);
}

/*********************************************************
    PROC:  getshort()
    DOES: 
**********************************************************/

unsigned short getshort(FILE *inf) 
{ 
  unsigned char buf[2]; 
 
  fread(buf,2,1,inf); 
  return (buf[0]<<8)+(buf[1]<<0); 
} 

/*********************************************************
    PROC:  getlong()
    DOES: 
**********************************************************/

long getlong(FILE *inf) 
{ 
  unsigned char buf[4]; 
 
  fread(buf,4,1,inf); 
  return (buf[0]<<24)+(buf[1]<<16)+(buf[2]<<8)+(buf[3]<<0); 
} 

/*********************************************************
    PROC:  putbyte()
    DOES: 
**********************************************************/

void putbyte(FILE *outf, unsigned char val) 
{ 
    unsigned char buf[1]; 

    buf[0] = val; 
        fwrite(buf,1,1,outf); 
    } 
 
/*********************************************************
    PROC:  putshort()
    DOES: 
**********************************************************/

void putshort(FILE *outf, unsigned short val) 
{ 
        unsigned char buf[2]; 
 
        buf[0] = (val>>8); 
        buf[1] = (val>>0); 
        fwrite(buf,2,1,outf); 
} 
 
/*********************************************************
    PROC:  putlong()
    DOES:  
**********************************************************/

static int putlong(FILE *outf, unsigned long val) 
{ 
	unsigned char buf[4]; 
 
	buf[0] = (unsigned char) (val>>24); 
	buf[1] = (unsigned char) (val>>16); 
	buf[2] = (unsigned char) (val>>8); 
	buf[3] = (unsigned char) (val>>0); 
	return fwrite(buf,4,1,outf); 
} 
 
/*********************************************************
    PROC:  find_image()
    DOES:  finds image of the given name
**********************************************************/

IMAGE *find_image(char *name)
{
  IMAGE *image;

  for (image=Image; image; image=image->next) {
    if (!strcmp(name,image->name))
      return(image);
  }
  return((IMAGE *) NULL);
}

/*********************************************************
    PROC:  newimage()
    DOES:  creates a new image
**********************************************************/

IMAGE *newimage(char *name)
{
  IMAGE *i;

  i = (IMAGE *) mymalloc(sizeof(IMAGE));
  strcpy(i->name,name);
  i->data = (unsigned char *) NULL;
  i->next = Image;

  Image = i;
  return(i);
}

/************************************************************
	PROC:	read_ppm_image()
	DOES:	reads a ppm image, sets it up to be an OpenGL texture
************************************************************/

void read_ppm_image(IMAGE *image, FILE *fp) {
	unsigned char c1, c2;

	fscanf(fp,"%c%c",&c1, &c2);
	if (c1!='P' || c2!='6') {
		printf("Unable to open file for texture '%s': not a PPM (raw) file (c1=%c, c2=%c).\n",
			image->name,c1,c2);
		return;
	}
	int xsize, ysize, maxval;
	fscanf(fp,"%d%d%d",&xsize, &ysize, &maxval);
	printf("%s: %d x %d image\n",image->name,xsize,ysize);
	image->xsize = xsize;
	image->ysize = ysize;
	fgetc(fp);           // discard white-space character
	if (image->data)     // discard old image, if any
		free(image->data);
	int size = xsize*ysize*3;
	image->data = (unsigned char *) malloc(size);
	int count=0;
	int rv = fread(image->data, 1, size, fp);
	return;
}

/************************************************************
	PROC:	print()
	DOES:	prints the contents of an image
************************************************************/

void print(int nargs, STR args[])
{
    int row;
    IMAGE *image;

    if (nargs!=3) {
      printf("Bad number of arguments\n");
      return;
    }
    image = find_image(args[1]);
    if (!image) {
      printf("Image '%s' not found\n",args[1]);
      return;
    }
	row = atoi(args[2]);
	if (row<0 || row>=image->ysize) {
	    printf("Error: the image only has %d scanlines\n",image->ysize);
	}
	prscanline(image,row);	    
}


/************************************************************
	PROC:	prscanline()
	DOES:	dumps the contents of a scanline
************************************************************/

void prscanline(IMAGE *image, int row)
{

}

/************************************************************
	PROC:   getpixel(image, u, v, r, g, b)
	DOES:	gets a pixel from one of the texture images
************************************************************/

void getpixel(IMAGE *i, float u, float v, float &r, float &g, float &b)
{
	if (i->clamp) {     // clamp out-of-bounds accesses
		if (u>1) u=1;
		if (u<0) u=0;
		if (v>1) v=1;
		if (v<0) v=0;
	} else {            // wrap-around out-of-bounds accesses
		if (u>1) u = u - int(u);
		else if (u<0) u = 1 + u - int(u);
		if (v>1) v = v - int(v);
		else if (v<0) v = 1 + v - int(v);
	}
	int x = int(u*(i->xsize - 1));
	int y = int(v*(i->ysize - 1));
	unsigned char *p = i->data + 3*(y * i->xsize + x);
	r = *p/255.0f;
	g = *(p+1)/255.0f;
	b = *(p+2)/255.0f;
}


/////////////////////////////////////////
//	PROC:   myBindTexture	
//	DOES:	
/////////////////////////////////////////

void myBindTexture(char *name)
{
	extern IMAGE *CurrTex;

	IMAGE *i = find_image(name);      // check to see if the image has been previously read in
	if (!i) {
		  FILE *fp = fopen(name,"r"); 
		  if(!fp) {
			  printf("Unable to open input file\n");
			  return;
		  }
		  IMAGE *image = newimage(name);
		  strcpy(image->name,name);
		  image->clamp = FALSE;
		  read_ppm_image(image, fp);
		  fclose(fp);
		  i = image;
	}
	CurrTex = i;
}




